
public class BinarySearchTree<E extends KeyedElementInterface<K>, K extends Comparable<? super K>> implements BinarySearchTreeInterface<E, K> {

	private TreeNode<E, K> root;
	
	public BinarySearchTree() {
		this.root = null;
	}
	
	public BinarySearchTree(TreeNode<E, K> root) {
		this.root = root;
	}
	
	@Override
	public TreeNode<E, K> getRoot() {
		return this.root;
	}

	@Override
	public void setRoot(TreeNode<E, K> root) {
		this.root = root;
	}

	@Override
	public boolean isEmpty() {
		return (this.root == null);
	}

	@Override
	public void makeEmpty() {
		this.root = null;
	}

	@Override
	public BinarySearchTreeInterface<E, K> copy() {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public E find(K key) {
		TreeNode<E, K> foundNode = this.findNode(key);
		if (foundNode != null) {
			return foundNode.getElement();
		} else {
			return null;
		}
	}

	@Override
	public void insert(E element) {
		// put element in a new node
		TreeNode<E, K> newNode = new TreeNode<E, K>(element);
		K elementKey = element.getKey();
		
		if (this.isEmpty()) {
			// the tree is empty, make the new node the root
			this.root = newNode;
		} else {
			// start at the root
			TreeNode<E, K> curNode = this.root;
			while (curNode != null) {
				K curKey = curNode.getElement().getKey();
				if (this.compareKeys(curKey, elementKey) > 0) {
					// the new element is smaller than the element in curNode
					if (curNode.getLeftChild() == null) {
						// curNode does not have a left child add the new node as its left child
						curNode.setLeftChild(newNode);
						newNode.setParent(curNode);
						return;
					} else {
						// make curNode's left child the new curNode
						curNode = curNode.getLeftChild();
					}
				} else {
					// the new element is greater than or equal the element at curNode
					if (curNode.getRightChild() == null) {
						// curNode does not have a right child add the new node as its right child
						curNode.setRightChild(newNode);
						newNode.setParent(curNode);
						return;
					} else {
						// make curNode's right child the new curNode
						curNode = curNode.getRightChild();
					}
				}
			}
		}
	}

	@Override
	public void delete(K key) throws TreeException {
		TreeNode<E, K> nodeToDelete = this.findNode(key);
		if (nodeToDelete != null) {
			this.deleteNode(nodeToDelete);
		}
	}

	@Override
	public int height() {
		// TODO Auto-generated method stub
		return 0;
	}

	@Override
	public boolean isBalanced() {
		// TODO Auto-generated method stub
		return false;
	}

	@Override
	public void balance() {
		// TODO Auto-generated method stub
		
	}
	
	public TreeElementIterator<E, K> iterator() {
		return new TreeElementIterator<E, K>(this);
	}


	/********************************************/
	/*                                          */
	/*        Private Utility Methods           */
	/*                                          */
	/********************************************/
	
	private int compareKeys(K key1, K key2) {
		// if key1 > key2 return positive number
		// if key1 == key2 return 0
		// if key1 < key2 return negative number
		return key1.compareTo(key2);
	}
	
	private TreeNode<E, K> findNode(K key) {
		TreeNode<E, K> curNode = this.root;
		
		while (curNode != null) {
			K curKey = curNode.getElement().getKey();
			if (this.compareKeys(curKey, key) == 0) {
				return curNode;
			} else if (this.compareKeys(curKey, key) > 0) {
				curNode = curNode.getLeftChild();
			} else {
				curNode = curNode.getRightChild();
			}
		}
		return null;
	}
	
	private void deleteNode(TreeNode<E, K> nodeToDelete) {
		TreeNode<E, K> parentNode = nodeToDelete.getParent();
		if (isLeafNode(nodeToDelete)) {
			// deleting a leaf node
			if (parentNode == null) {
				// the leaf is also the root and only node in the tree
				this.root = null;
			} else {
				if (this.isNodeLeftChildOfParentNode(nodeToDelete, parentNode)) {
					// the node to delete is the left child of the parent
					parentNode.setLeftChild(null);
				} else {
					// the node to delete is the right  child of the parent
					parentNode.setRightChild(null);
				}
			}
		} else if (this.nodeOnlyHasLeftChild(nodeToDelete)) {
			if (parentNode == null) {
				// the node being deleted is the root
				// promote its left child to be the root
				this.root = nodeToDelete.getLeftChild();
				this.root.setParent(null);
			} else if (this.isNodeLeftChildOfParentNode(nodeToDelete, parentNode)) {
				// the node to delete is the left child of parent
				parentNode.setLeftChild(nodeToDelete.getLeftChild());
				nodeToDelete.getLeftChild().setParent(parentNode);
			} else {
				// the node to delete is the right child of parent
				parentNode.setRightChild(nodeToDelete.getLeftChild());
				nodeToDelete.getLeftChild().setParent(parentNode);
			}
		} else if (this.nodeOnlyHasRightChild(nodeToDelete)) {
			if (parentNode == null) {
				// the node being deleted is the root
				// promote its left child to be the root
				this.root = nodeToDelete.getRightChild();
				this.root.setParent(null);
			} else if (this.isNodeLeftChildOfParentNode(nodeToDelete, parentNode)) {
				// the node being deleted is the left child of parent
				parentNode.setLeftChild(nodeToDelete.getRightChild());
				nodeToDelete.getRightChild().setParent(parentNode);
			} else {
				// the node being deleted is the right child of parent
				parentNode.setRightChild(nodeToDelete.getRightChild());
				nodeToDelete.getRightChild().setParent(parentNode);
			}
		} else {
			// node being deleted has 2 children
			TreeNode<E, K> successor = this.findSuccessorNode(nodeToDelete);
			// move the element from the successor to the node being deleted
			nodeToDelete.setElement(successor.getElement());
			// delete the successor node
			this.deleteNode(successor);
		}
		
	}
	
	private boolean isLeafNode(TreeNode<E, K> node) {
		if ((node.getLeftChild() == null) && (node.getRightChild() == null)) {
			return true;
		} else {
			return false;
		}
	}
	
	private boolean isNodeLeftChildOfParentNode(TreeNode<E, K> node, TreeNode<E, K> parent) {
		return (parent.getLeftChild() == node);
	}
	
	private boolean nodeOnlyHasLeftChild(TreeNode<E, K> node) {
		return ((node.getLeftChild() != null) && (node.getRightChild() == null));
	}
	
	private boolean nodeOnlyHasRightChild(TreeNode<E, K> node) {
		return ((node.getLeftChild() == null) && (node.getRightChild() != null));
	}
	
	private TreeNode<E, K> findSuccessorNode(TreeNode<E, K> node) {
		TreeNode<E, K> curNode = node.getRightChild();
		while (curNode.getLeftChild() != null) {
			curNode = curNode.getLeftChild();
		}
		return curNode;
	}
}
