import java.util.Iterator;
import java.util.NoSuchElementException;
import java.util.Vector;

public class TreeElementIterator<E extends KeyedElementInterface<K>, K extends Comparable<? super K>> implements Iterator<E> {

	private BinarySearchTree<E, K> bst;
	
	private Vector<TreeNode<E, K>> treeNodes;
	
	
	
	public TreeElementIterator(BinarySearchTree<E, K> bst) {
		this.bst = bst;
		this.setInOrder();
	}
	
	public int size() {
		if (this.treeNodes != null) {
			return this.treeNodes.size();
		} else {
			return 0;
		}
	}
	
	public boolean isEmpty() {
		if (this.treeNodes == null) {
			return true;
		} else {
			return (this.treeNodes.size() == 0);
		}
	}
	
	public void setInOrder() {
		this.treeNodes = new Vector<TreeNode<E, K>>();
		this.inOrder(bst.getRoot());
	}
	
	public void setPreOrder() {
		this.treeNodes = new Vector<TreeNode<E, K>>();
		this.preOrder(bst.getRoot());
	}
	
	public void setPostOrder() {
		this.treeNodes = new Vector<TreeNode<E, K>>();
		this.postOrder(bst.getRoot());
	}
	
	@Override
	public boolean hasNext() {
		if (this.isEmpty()) {
			return false;
		} else {
			return true;
		}
	}

	@Override
	public E next() throws NoSuchElementException {
		if (this.isEmpty()) {
			this.treeNodes = null;
			throw new NoSuchElementException("Iterator has no Elements to return");
		} else {
			E element = this.treeNodes.get(0).getElement();
			this.treeNodes.remove(0);
			return element;
		}
	}

	
	
	
	
	
	
	
	
	
	private void inOrder(TreeNode<E, K> node) {
		if (node == null) {
			return;
		}
		
		this.inOrder(node.getLeftChild());
		this.treeNodes.add(node);
		this.inOrder(node.getRightChild());
	}
	
	private void preOrder(TreeNode<E, K> node) {
		if (node == null) {
			return;
		}
		
		this.treeNodes.add(node);
		this.preOrder(node.getLeftChild());
		this.preOrder(node.getRightChild());
	}

	
	private void postOrder(TreeNode<E, K> node) {
		if (node == null) {
			return;
		}
		
		this.postOrder(node.getLeftChild());
		this.postOrder(node.getRightChild());
		this.treeNodes.add(node);
	}
}
