
/**
 * 
 * @author sameh
 *
 * @param <E> The type of <code>element</code> that the <code>BinarySearchTree</code> will hold. Please note that <code>E</code> must implement the
 * <code>KeyElementInterface</code> or some super class of <code>E</code> must  implement the <code>KeyElementInterface</code>.
 *     
 * @param <K> The type of <code>key</code> that all <code>element</code>s must contain. Please note that <code>K</code> must implement the
 * <code>Comparable</code> interface or some super class of <code>K</code> must  implement the <code>Comparable</code> interface.
 */
public class TreeNode<E extends KeyedElementInterface<K>, K extends Comparable<? super K>> {

	/**
	 * This instance variable holds the element that is in the BinarySearchTree
	 */
	private E element;
	
	/**
	 * This instance variable is a reference to the left child of the current <code>TreeNode</code>
	 */
	private TreeNode<E,K> leftChild;
	
	/**
	 * This instance variable is a reference to the right child of the current <code>TreeNode</code>
	 */
	private TreeNode<E,K> rightChild;
	
	/**
	 * This instance variable is a reference to the parent of the current <code>TreeNode</code>
	 */
	private TreeNode<E,K> parent;
	
	/**
	 * 
	 * Constructor for the <code>TreeNode</code>. This constructor should set <code>leftChild</code>,
	 * <code>rightChild</code>, and <code>parent</code> to <code>null</code>. 
	 * 
	 * @param element This is parameter is used to set the <code>element</code> instance variable.
	 * 
	 */
	public TreeNode(E element) {
		this.element = element;
		this.leftChild = null;
		this.rightChild = null;
		this.parent = null;
	}

	/**
	 * 
	 * Getter for <code>leftChild</code> instance variable.
	 * 
	 * @return The <code>leftChild</code> of the current <code>TreeNode</code>
	 *  
	 */
	public TreeNode<E, K> getLeftChild() {
		return leftChild;
	}

	/**
	 * 
	 * Setter for <code>leftChild</code> instance variable.
	 * 
	 * @param leftChild The new <code>leftChild</code> of the current <code>TreeNode</code>
	 *   
	 */
	public void setLeftChild(TreeNode<E, K> leftChild) {
		this.leftChild = leftChild;
	}

	/**
	 * 
	 * Getter for <code>rightChild</code> instance variable.
	 * 
	 * @return The <code>rightChild</code> of the current <code>TreeNode</code>
	 *  
	 */
	public TreeNode<E, K> getRightChild() {
		return rightChild;
	}

	/**
	 * 
	 * Setter for <code>rightChild</code> instance variable.
	 * 
	 * @param rightChild The new <code>rightChild</code> of the current <code>TreeNode</code>
	 *   
	 */
	public void setRightChild(TreeNode<E, K> rightChild) {
		this.rightChild = rightChild;
	}

	/**
	 * 
	 * Getter for <code>parent</code> instance variable.
	 * 
	 * @return The <code>parent</code> of the current <code>TreeNode</code>
	 *  
	 */
	public TreeNode<E, K> getParent() {
		return parent;
	}

	/**
	 * 
	 * Setter for <code>parent</code> instance variable.
	 * 
	 * @param parent The new <code>parent</code> of the current <code>TreeNode</code>
	 *   
	 */
	public void setParent(TreeNode<E, K> parent) {
		this.parent = parent;
	}

	/**
	 * 
	 * Getter for <code>element</code> instance variable.
	 * 
	 * @return The <code>element</code> of the current <code>TreeNode</code>
	 *  
	 */
	public E getElement() {
		return element;
	}

	/**
	 * 
	 * Setter for <code>element</code> instance variable.
	 * 
	 * @param element The new <code>element</code> of the current <code>TreeNode</code>
	 *   
	 */
	public void setElement(E element) {
		this.element = element;
	}
}
